@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Description: MySQL netdata python.d module
 # Author: Pawel Krupa (paulfantom)
-# Author: Ilya Mashchenko (l2isbad)
+# Author: Ilya Mashchenko (ilyam8)
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 from bases.FrameworkServices.MySQLService import MySQLService
@@ -11,6 +11,7 @@ from bases.FrameworkServices.MySQLService import MySQLService
 QUERY_GLOBAL = 'SHOW GLOBAL STATUS;'
 QUERY_SLAVE = 'SHOW SLAVE STATUS;'
 QUERY_VARIABLES = 'SHOW GLOBAL VARIABLES LIKE \'max_connections\';'
+QUERY_USER_STATISTICS = 'SHOW USER_STATISTICS;'
 
 GLOBAL_STATS = [
     'Bytes_received',
@@ -90,6 +91,7 @@ GLOBAL_STATS = [
     'Innodb_buffer_pool_write_requests',
     'Innodb_buffer_pool_reads',
     'Innodb_buffer_pool_wait_free',
+    'Innodb_deadlocks',
     'Qcache_hits',
     'Qcache_lowmem_prunes',
     'Qcache_inserts',
@@ -115,6 +117,14 @@ GLOBAL_STATS = [
     'Connection_errors_peer_address',
     'Connection_errors_select',
     'Connection_errors_tcpwrap',
+    'Com_delete',
+    'Com_insert',
+    'Com_select',
+    'Com_update',
+    'Com_replace'
+]
+
+GALERA_STATS = [
     'wsrep_local_recv_queue',
     'wsrep_local_send_queue',
     'wsrep_received',
@@ -124,11 +134,14 @@ GLOBAL_STATS = [
     'wsrep_local_bf_aborts',
     'wsrep_local_cert_failures',
     'wsrep_flow_control_paused_ns',
-    'Com_delete',
-    'Com_insert',
-    'Com_select',
-    'Com_update',
-    'Com_replace'
+    'wsrep_cluster_weight',
+    'wsrep_cluster_size',
+    'wsrep_cluster_status',
+    'wsrep_local_state',
+    'wsrep_open_transactions',
+    'wsrep_connected',
+    'wsrep_ready',
+    'wsrep_thread_count'
 ]
 
 
@@ -149,6 +162,18 @@ SLAVE_STATS = [
     ('Slave_IO_Running', slave_running)
 ]
 
+USER_STATISTICS = [
+    'Select_commands',
+    'Update_commands',
+    'Other_commands',
+    'Cpu_time',
+    'Rows_read',
+    'Rows_sent',
+    'Rows_deleted',
+    'Rows_inserted',
+    'Rows_updated'
+]
+
 VARIABLES = [
     'max_connections'
 ]
@@ -178,6 +203,7 @@ ORDER = [
     'innodb_os_log_fsync_writes',
     'innodb_os_log_io',
     'innodb_cur_row_lock',
+    'innodb_deadlocks',
     'innodb_rows',
     'innodb_buffer_pool_pages',
     'innodb_buffer_pool_flush_pages_requests',
@@ -200,7 +226,16 @@ ORDER = [
     'galera_bytes',
     'galera_queue',
     'galera_conflicts',
-    'galera_flow_control'
+    'galera_flow_control',
+    'galera_cluster_status',
+    'galera_cluster_state',
+    'galera_cluster_size',
+    'galera_cluster_weight',
+    'galera_connected',
+    'galera_ready',
+    'galera_open_transactions',
+    'galera_thread_count',
+    'userstats_cpu',
 ]
 
 CHARTS = {
@@ -313,7 +348,7 @@ CHARTS = {
         ]
     },
     'threads_creation_rate': {
-        'options': [None, 'Threads Creation Rate', 'threads', 'threads/s', 'mysql.threads', 'line'],
+        'options': [None, 'Threads Creation Rate', 'threads/s', 'threads', 'mysql.threads', 'line'],
         'lines': [
             ['Threads_created', 'created', 'incremental'],
         ]
@@ -382,6 +417,13 @@ CHARTS = {
             ['Innodb_row_lock_current_waits', 'current_waits', 'absolute']
         ]
     },
+    'innodb_deadlocks': {
+        'options': [None, 'InnoDB Deadlocks', 'operations/s', 'innodb',
+                    'mysql.innodb_deadlocks', 'area'],
+        'lines': [
+            ['Innodb_deadlocks', 'deadlocks', 'incremental']
+        ]
+    },
     'innodb_rows': {
         'options': [None, 'InnoDB Row Operations', 'operations/s', 'innodb', 'mysql.innodb_rows', 'area'],
         'lines': [
@@ -570,10 +612,181 @@ CHARTS = {
         'lines': [
             ['wsrep_flow_control_paused_ns', 'paused', 'incremental', 1, 1000000],
         ]
+    },
+    'galera_cluster_status': {
+        'options': [None, 'Cluster Component Status', 'status', 'galera', 'mysql.galera_cluster_status', 'line'],
+        'lines': [
+            ['wsrep_cluster_status', 'status', 'absolute'],
+        ]
+    },
+    'galera_cluster_state': {
+        'options': [None, 'Cluster Component State', 'state', 'galera', 'mysql.galera_cluster_state', 'line'],
+        'lines': [
+            ['wsrep_local_state', 'state', 'absolute'],
+        ]
+    },
+    'galera_cluster_size': {
+        'options': [None, 'Number of Nodes in the Cluster', 'num', 'galera', 'mysql.galera_cluster_size', 'line'],
+        'lines': [
+            ['wsrep_cluster_size', 'nodes', 'absolute'],
+        ]
+    },
+    'galera_cluster_weight': {
+        'options': [None, 'The Total Weight of the Current Members in the Cluster', 'weight', 'galera',
+                    'mysql.galera_cluster_weight', 'line'],
+        'lines': [
+            ['wsrep_cluster_weight', 'weight', 'absolute'],
+        ]
+    },
+    'galera_connected': {
+        'options': [None, 'Whether the Node is Connected to the Cluster', 'boolean', 'galera',
+                    'mysql.galera_connected', 'line'],
+        'lines': [
+            ['wsrep_connected', 'connected', 'absolute'],
+        ]
+    },
+    'galera_ready': {
+        'options': [None, 'Whether the Node is Ready to Accept Queries', 'boolean', 'galera',
+                    'mysql.galera_ready', 'line'],
+        'lines': [
+            ['wsrep_ready', 'ready', 'absolute'],
+        ]
+    },
+    'galera_open_transactions': {
+        'options': [None, 'Open Transactions', 'num', 'galera', 'mysql.galera_open_transactions', 'line'],
+        'lines': [
+            ['wsrep_open_transactions', 'open transactions', 'absolute'],
+        ]
+    },
+    'galera_thread_count': {
+        'options': [None, 'Total Number of WSRep (applier/rollbacker) Threads', 'num', 'galera',
+                    'mysql.galera_thread_count', 'line'],
+        'lines': [
+            ['wsrep_thread_count', 'threads', 'absolute'],
+        ]
+    },
+    'userstats_cpu': {
+        'options': [None, 'Users CPU time', 'percentage', 'userstats', 'mysql.userstats_cpu', 'stacked'],
+        'lines': []
     }
 }
 
 
+def slave_status_chart_template(channel_name):
+    order = [
+        'slave_behind_{0}'.format(channel_name),
+        'slave_status_{0}'.format(channel_name)
+    ]
+
+    charts = {
+        order[0]: {
+            'options': [None, 'Slave Behind Seconds Channel {0}'.format(channel_name),
+                        'seconds', 'slave', 'mysql.slave_behind', 'line'],
+            'lines': [
+                ['Seconds_Behind_Master_{0}'.format(channel_name), 'seconds', 'absolute']
+            ]
+        },
+        order[1]: {
+            'options': [None, 'Slave Status Channel {0}'.format(channel_name),
+                        'status', 'slave', 'mysql.slave_status', 'line'],
+            'lines': [
+                ['Slave_SQL_Running_{0}'.format(channel_name), 'sql_running', 'absolute'],
+                ['Slave_IO_Running_{0}'.format(channel_name), 'io_running', 'absolute']
+            ]
+        },
+    }
+
+    return order, charts
+
+
+def userstats_chart_template(name):
+    order = [
+        'userstats_rows_{0}'.format(name),
+        'userstats_commands_{0}'.format(name)
+    ]
+    family = 'userstats {0}'.format(name)
+
+    charts = {
+        order[0]: {
+            'options': [None, 'Rows Operations', 'operations/s', family, 'mysql.userstats_rows', 'stacked'],
+            'lines': [
+                ['userstats_{0}_Rows_read'.format(name), 'read', 'incremental'],
+                ['userstats_{0}_Rows_send'.format(name), 'send', 'incremental'],
+                ['userstats_{0}_Rows_updated'.format(name), 'updated', 'incremental'],
+                ['userstats_{0}_Rows_inserted'.format(name), 'inserted', 'incremental'],
+                ['userstats_{0}_Rows_deleted'.format(name), 'deleted', 'incremental']
+            ]
+        },
+        order[1]: {
+            'options': [None, 'Commands', 'commands/s', family, 'mysql.userstats_commands', 'stacked'],
+            'lines': [
+                ['userstats_{0}_Select_commands'.format(name), 'select', 'incremental'],
+                ['userstats_{0}_Update_commands'.format(name), 'update', 'incremental'],
+                ['userstats_{0}_Other_commands'.format(name), 'other', 'incremental']
+            ]
+        }
+    }
+
+    return order, charts
+
+
+# https://dev.mysql.com/doc/refman/8.0/en/replication-channels.html
+DEFAULT_REPL_CHANNEL = ''
+
+
+# Write Set REPlication
+# https://galeracluster.com/library/documentation/galera-status-variables.html
+# https://www.percona.com/doc/percona-xtradb-cluster/LATEST/wsrep-status-index.html
+class WSRepDataConverter:
+    unknown_value = -1
+
+    def convert(self, key, value):
+        if key == 'wsrep_connected':
+            return self.convert_connected(value)
+        elif key == 'wsrep_ready':
+            return self.convert_ready(value)
+        elif key == 'wsrep_cluster_status':
+            return self.convert_cluster_status(value)
+        return value
+
+    def convert_connected(self, value):
+        # https://www.percona.com/doc/percona-xtradb-cluster/LATEST/wsrep-status-index.html#wsrep_connected
+        if value == 'OFF':
+            return 0
+        if value == 'ON':
+            return 1
+        return self.unknown_value
+
+    def convert_ready(self, value):
+        # https://www.percona.com/doc/percona-xtradb-cluster/LATEST/wsrep-status-index.html#wsrep_ready
+        if value == 'OFF':
+            return 0
+        if value == 'ON':
+            return 1
+        return self.unknown_value
+
+    def convert_cluster_status(self, value):
+        # https://www.percona.com/doc/percona-xtradb-cluster/LATEST/wsrep-status-index.html#wsrep_cluster_status
+        # https://github.com/codership/wsrep-API/blob/eab2d5d5a31672c0b7d116ef1629ff18392fd7d0/wsrep_api.h
+        # typedef enum wsrep_view_status {
+        #     WSREP_VIEW_PRIMARY,      //!< primary group configuration (quorum present)
+        #     WSREP_VIEW_NON_PRIMARY,  //!< non-primary group configuration (quorum lost)
+        #     WSREP_VIEW_DISCONNECTED, //!< not connected to group, retrying.
+        #     WSREP_VIEW_MAX
+        # } wsrep_view_status_t;
+        value = value.lower()
+        if value == 'primary':
+            return 0
+        elif value == 'non-primary':
+            return 1
+        elif value == 'disconnected':
+            return 2
+        return self.unknown_value
+
+
+wsrep_converter = WSRepDataConverter()
+
+
 class Service(MySQLService):
     def __init__(self, configuration=None, name=None):
         MySQLService.__init__(self, configuration=configuration, name=name)
@@ -583,7 +796,9 @@ class Service(MySQLService):
             global_status=QUERY_GLOBAL,
             slave_status=QUERY_SLAVE,
             variables=QUERY_VARIABLES,
+            user_statistics=QUERY_USER_STATISTICS,
         )
+        self.repl_channels = [DEFAULT_REPL_CHANNEL]
 
     def _get_data(self):
 
@@ -592,30 +807,171 @@ class Service(MySQLService):
         if not raw_data:
             return None
 
-        to_netdata = dict()
+        data = dict()
 
         if 'global_status' in raw_data:
-            global_status = dict(raw_data['global_status'][0])
-            for key in GLOBAL_STATS:
-                if key in global_status:
-                    to_netdata[key] = global_status[key]
-            if 'Threads_created' in to_netdata and 'Connections' in to_netdata:
-                to_netdata['Thread_cache_misses'] = round(int(to_netdata['Threads_created'])
-                                                          / float(to_netdata['Connections']) * 10000)
+            global_status = self.get_global_status(raw_data['global_status'])
+            if global_status:
+                data.update(global_status)
 
         if 'slave_status' in raw_data:
-            if raw_data['slave_status'][0]:
-                slave_raw_data = dict(zip([e[0] for e in raw_data['slave_status'][1]], raw_data['slave_status'][0][0]))
-                for key, func in SLAVE_STATS:
-                    if key in slave_raw_data:
-                        to_netdata[key] = func(slave_raw_data[key])
+            status = self.get_slave_status(raw_data['slave_status'])
+            if status:
+                data.update(status)
+
+        if 'user_statistics' in raw_data:
+            if raw_data['user_statistics'][0]:
+                data.update(self.get_userstats(raw_data))
             else:
-                self.queries.pop('slave_status')
+                self.queries.pop('user_statistics')
 
         if 'variables' in raw_data:
             variables = dict(raw_data['variables'][0])
             for key in VARIABLES:
                 if key in variables:
-                    to_netdata[key] = variables[key]
+                    data[key] = variables[key]
 
-        return to_netdata or None
+        return data or None
+
+    @staticmethod
+    def convert_wsrep(key, value):
+        return wsrep_converter.convert(key, value)
+
+    def get_global_status(self, raw_global_status):
+        # (
+        #     (
+        #         ('Aborted_clients', '18'),
+        #         ('Aborted_connects', '33'),
+        #         ('Access_denied_errors', '80'),
+        #         ('Acl_column_grants', '0'),
+        #         ('Acl_database_grants', '0'),
+        #         ('Acl_function_grants', '0'),
+        #         ('wsrep_ready', 'OFF'),
+        #         ('wsrep_rollbacker_thread_count', '0'),
+        #         ('wsrep_thread_count', '0')
+        #     ),
+        #     (
+        #         ('Variable_name', 253, 60, 64, 64, 0, 0),
+        #         ('Value', 253, 48, 2048, 2048, 0, 0),
+        #     )
+        # )
+        rows = raw_global_status[0]
+        if not rows:
+            return
+
+        global_status = dict(rows)
+        data = dict()
+
+        for key in GLOBAL_STATS:
+            if key not in global_status:
+                continue
+            value = global_status[key]
+            data[key] = value
+
+        for key in GALERA_STATS:
+            if key not in global_status:
+                continue
+            value = global_status[key]
+            value = self.convert_wsrep(key, value)
+            data[key] = value
+
+        if 'Threads_created' in data and 'Connections' in data:
+            data['Thread_cache_misses'] = round(int(data['Threads_created']) / float(data['Connections']) * 10000)
+        return data
+
+    def get_slave_status(self, slave_status_data):
+        rows, description = slave_status_data[0], slave_status_data[1]
+        description_keys = [v[0] for v in description]
+        if not rows:
+            return
+
+        data = dict()
+        for row in rows:
+            slave_data = dict(zip(description_keys, row))
+            channel_name = slave_data.get('Channel_Name', DEFAULT_REPL_CHANNEL)
+
+            if channel_name not in self.repl_channels and len(self.charts) > 0:
+                self.add_repl_channel_charts(channel_name)
+                self.repl_channels.append(channel_name)
+
+            for key, func in SLAVE_STATS:
+                if key not in slave_data:
+                    continue
+
+                value = slave_data[key]
+                if channel_name:
+                    key = '{0}_{1}'.format(key, channel_name)
+                data[key] = func(value)
+
+        return data
+
+    def add_repl_channel_charts(self, name):
+        self.add_new_charts(slave_status_chart_template, name)
+
+    def get_userstats(self, raw_data):
+        # (
+        #     (
+        #         ('netdata', 1L, 0L, 60L, 0.15842499999999984, 0.15767439999999996, 5206L, 963957L, 0L, 0L,
+        #          61L, 0L, 0L, 0L, 0L, 0L, 62L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L),
+        #     ),
+        #     (
+        #         ('User', 253, 7, 128, 128, 0, 0),
+        #         ('Total_connections', 3, 2, 11, 11, 0, 0),
+        #         ('Concurrent_connections', 3, 1, 11, 11, 0, 0),
+        #         ('Connected_time', 3, 2, 11, 11, 0, 0),
+        #         ('Busy_time', 5, 20, 21, 21, 31, 0),
+        #         ('Cpu_time', 5, 20, 21, 21, 31, 0),
+        #         ('Bytes_received', 8, 4, 21, 21, 0, 0),
+        #         ('Bytes_sent', 8, 6, 21, 21, 0, 0),
+        #         ('Binlog_bytes_written', 8, 1, 21, 21, 0, 0),
+        #         ('Rows_read', 8, 1, 21, 21, 0, 0),
+        #         ('Rows_sent', 8, 2, 21, 21, 0, 0),
+        #         ('Rows_deleted', 8, 1, 21, 21, 0, 0),
+        #         ('Rows_inserted', 8, 1, 21, 21, 0, 0),
+        #         ('Rows_updated', 8, 1, 21, 21, 0, 0),
+        #         ('Select_commands', 8, 2, 21, 21, 0, 0),
+        #         ('Update_commands', 8, 1, 21, 21, 0, 0),
+        #         ('Other_commands', 8, 2, 21, 21, 0, 0),
+        #         ('Commit_transactions', 8, 1, 21, 21, 0, 0),
+        #         ('Rollback_transactions', 8, 1, 21, 21, 0, 0),
+        #         ('Denied_connections', 8, 1, 21, 21, 0, 0),
+        #         ('Lost_connections', 8, 1, 21, 21, 0, 0),
+        #         ('Access_denied', 8, 1, 21, 21, 0, 0),
+        #         ('Empty_queries', 8, 2, 21, 21, 0, 0),
+        #         ('Total_ssl_connections', 8, 1, 21, 21, 0, 0),
+        #         ('Max_statement_time_exceeded', 8, 1, 21, 21, 0, 0)
+        #     )
+        # )
+        data = dict()
+        userstats_vars = [e[0] for e in raw_data['user_statistics'][1]]
+        for i, _ in enumerate(raw_data['user_statistics'][0]):
+            user_name = raw_data['user_statistics'][0][i][0]
+            userstats = dict(zip(userstats_vars, raw_data['user_statistics'][0][i]))
+
+            if len(self.charts) > 0:
+                if ('userstats_{0}_Cpu_time'.format(user_name)) not in self.charts['userstats_cpu']:
+                    self.add_userstats_dimensions(user_name)
+                    self.create_new_userstats_charts(user_name)
+
+            for key in USER_STATISTICS:
+                if key in userstats:
+                    data['userstats_{0}_{1}'.format(user_name, key)] = userstats[key]
+
+        return data
+
+    def add_userstats_dimensions(self, name):
+        self.charts['userstats_cpu'].add_dimension(['userstats_{0}_Cpu_time'.format(name), name, 'incremental', 100, 1])
+
+    def create_new_userstats_charts(self, tube):
+        self.add_new_charts(userstats_chart_template, tube)
+
+    def add_new_charts(self, template, *params):
+        order, charts = template(*params)
+
+        for chart_name in order:
+            params = [chart_name] + charts[chart_name]['options']
+            dimensions = charts[chart_name]['lines']
+
+            new_chart = self.charts.add_chart(params)
+            for dimension in dimensions:
+                new_chart.add_dimension(dimension)
