@@ -3,12 +3,13 @@
 # docker run -it --rm centos:6.9 /bin/sh
 # yum -y install rpm-build redhat-rpm-config yum-utils autoconf automake curl gcc git libmnl-devel libuuid-devel make pkgconfig zlib-devel
 
-cd $(dirname $0)/../../ || exit 1
-source "installer/functions.sh" || exit 1
+cd "$(dirname "$0")/../../" || exit 1
+# shellcheck disable=SC1091
+source "packaging/installer/functions.sh" || exit 1
 
 set -e
 
-run ./autogen.sh
+run autoreconf -ivf
 run ./configure --enable-maintainer-mode
 run make dist
 
@@ -27,7 +28,7 @@ then
 fi
 
 srpm=$(run rpmbuild -ts "${tgz}" | cut -d ' ' -f 2)
-if [ -z "${srpm}" -o ! -f "${srpm}" ]
+if [ -z "${srpm}" ] || [ ! -f "${srpm}" ]
 then
 	echo >&2 "Cannot find the generated SRPM file '${srpm}'"
 	exit 1
