@@ -11,6 +11,9 @@ extern int unit_test_buffer(void);
 #ifdef ENABLE_DBENGINE
 extern int test_dbengine(void);
 extern void generate_dbengine_dataset(unsigned history_seconds);
+extern void dbengine_stress_test(unsigned TEST_DURATION_SEC, unsigned DSET_CHARTS, unsigned QUERY_THREADS,
+                                 unsigned RAMP_UP_SECONDS, unsigned PAGE_CACHE_MB);
+
 #endif
 
 #endif /* NETDATA_UNIT_TEST_H */
