@@ -148,6 +148,7 @@ struct rrdengine_statistics {
     rrdeng_stats_t page_cache_descriptors;
     rrdeng_stats_t io_errors;
     rrdeng_stats_t fs_errors;
+    rrdeng_stats_t flushing_errors;
 };
 
 /* I/O errors global counter */
@@ -156,6 +157,8 @@ extern rrdeng_stats_t global_io_errors;
 extern rrdeng_stats_t global_fs_errors;
 /* number of File-Descriptors that have been reserved by dbengine */
 extern rrdeng_stats_t rrdeng_reserved_file_descriptors;
+/* inability to flush global counter */
+extern rrdeng_stats_t global_flushing_errors;
 
 struct rrdengine_instance {
     struct rrdengine_worker_config worker_config;
