@@ -104,7 +104,8 @@ markdown_extensions:
  - pymdownx.details
  - pymdownx.highlight:
     pygments_style: manni
-    noclasses: true
+    css_class: "highlight codehilite"
+    linenums_style: pymdownx-inline
  - pymdownx.inlinehilite
  - pymdownx.magiclink
  - pymdownx.mark
@@ -135,7 +136,6 @@ echo -ne "    - 'docs/what-is-netdata.md'
     - 'docs/a-github-star-is-important.md'
     - REDISTRIBUTED.md
     - CHANGELOG.md
-    - CONTRIBUTING.md
     - SECURITY.md
 - Why Netdata:
     - 'docs/why-netdata/README.md'
@@ -149,12 +149,11 @@ echo -ne "    - 'docs/what-is-netdata.md'
     - 'packaging/installer/UPDATE.md'
     - 'packaging/DISTRIBUTIONS.md'
     - 'packaging/installer/UNINSTALL.md'
-- 'docs/GettingStarted.md'
+- 'docs/getting-started.md'
 - Running Netdata:
     - 'daemon/README.md'
     - 'docs/configuration-guide.md'
     - 'daemon/config/README.md'
-    - 'docs/Charts.md'
 "
 navpart 2 web/server "" "Web server"
 navpart 3 web/server "" "" 2 excludefirstlevel
@@ -175,6 +174,16 @@ echo -ne "    - 'docs/Performance.md'
     - 'docs/high-performance-netdata.md'
 "
 
+navpart 1 . netdata-cloud "Netdata Cloud"
+echo -ne "
+    - 'docs/netdata-cloud/README.md'
+    - 'docs/netdata-cloud/signing-in.md'
+    - 'docs/netdata-cloud/nodes-view.md'
+"
+
+navpart 1 web "README" "Dashboards"
+navpart 2 web/gui "" "" 3
+
 navpart 1 collectors "" "Data collection" 1
 echo -ne "    - 'docs/Add-more-charts-to-netdata.md'
     - Internal plugins:
@@ -241,6 +250,7 @@ navpart 3 collectors/freeipmi.plugin
 navpart 3 collectors/nfacct.plugin
 navpart 3 collectors/xenstat.plugin
 navpart 3 collectors/perf.plugin
+navpart 3 collectors/slabinfo.plugin
 
 
 echo -ne "    - 'docs/Third-Party-Plugins.md'
@@ -254,9 +264,6 @@ navpart 1 streaming "" "" 4
 
 navpart 1 backends "" "Archiving to backends" 3
 
-navpart 1 web "README" "Dashboards"
-navpart 2 web/gui "" "" 3
-
 navpart 1 web/api "" "HTTP API"
 navpart 2 web/api/exporters "" "Exporters" 2
 navpart 2 web/api/formatters "" "Formatters" 2
@@ -264,11 +271,17 @@ navpart 2 web/api/badges "" "" 2
 navpart 2 web/api/health "" "" 2
 navpart 2 web/api/queries "" "Queries" 2
 
-echo -ne "- Additional Info:
+echo -ne "- Contributing to Netdata:
+    - CONTRIBUTING.md
+    - 'docs/contributing/contributing-documentation.md'
+    - 'docs/contributing/style-guide.md'
     - CODE_OF_CONDUCT.md
     - CONTRIBUTORS.md
     - packaging/maintainers/README.md
 "
+
+echo -ne "- Additional information:
+"
 navpart 2 packaging/makeself "" "" 4
 navpart 2 libnetdata "" "libnetdata" 4
 navpart 2 contrib
