@@ -411,6 +411,27 @@ int appconfig_set_boolean(struct config *root, const char *section, const char *
     return value;
 }
 
+int appconfig_get_duration(struct config *root, const char *section, const char *name, const char *value)
+{
+    int result = 0;
+    const char *s;
+
+    s = appconfig_get(root, section, name, value);
+    if(!s) goto fallback;
+
+    if(!config_parse_duration(s, &result)) {
+        error("config option '[%s].%s = %s' is configured with an valid duration", section, name, s);
+        goto fallback;
+    }
+
+    return result;
+
+    fallback:
+    if(!config_parse_duration(value, &result))
+        error("INTERNAL ERROR: default duration supplied for option '[%s].%s = %s' is not a valid duration", section, name, value);
+
+    return result;
+}
 
 // ----------------------------------------------------------------------------
 // config load/save
@@ -586,3 +607,65 @@ void appconfig_generate(struct config *root, BUFFER *wb, int only_changed)
         appconfig_unlock(root);
     }
 }
+
+/**
+ * Parse Duration
+ *
+ * Parse the string setting the result
+ *
+ * @param string  the timestamp string
+ * @param result the output variable
+ *
+ * @return It returns 1 on success and 0 otherwise
+ */
+int config_parse_duration(const char* string, int* result) {
+    while(*string && isspace(*string)) string++;
+
+    if(unlikely(!*string)) goto fallback;
+
+    if(*string == 'n' && !strcmp(string, "never")) {
+        // this is a valid option
+        *result = 0;
+        return 1;
+    }
+
+    // make sure it is a number
+    if(!(isdigit(*string) || *string == '+' || *string == '-')) goto fallback;
+
+    char *e = NULL;
+    calculated_number n = str2ld(string, &e);
+    if(e && *e) {
+        switch (*e) {
+            case 'Y':
+                *result = (int) (n * 31536000);
+                break;
+            case 'M':
+                *result = (int) (n * 2592000);
+                break;
+            case 'w':
+                *result = (int) (n * 604800);
+                break;
+            case 'd':
+                *result = (int) (n * 86400);
+                break;
+            case 'h':
+                *result = (int) (n * 3600);
+                break;
+            case 'm':
+                *result = (int) (n * 60);
+                break;
+            case 's':
+            default:
+                *result = (int) (n);
+                break;
+        }
+    }
+    else
+        *result = (int)(n);
+
+    return 1;
+
+    fallback:
+    *result = 0;
+    return 0;
+}
